@@ -7,6 +7,7 @@ from six.moves.urllib.parse import urlunparse
 
 from rq import Queue
 
+from dallinger.command_line.utils import error
 from dallinger.command_line.utils import header
 from dallinger.command_line.utils import log
 from dallinger.command_line.utils import Output
@@ -15,6 +16,7 @@ from dallinger.config import get_config
 from dallinger.db import redis_conn
 from dallinger.deployment import DevelopmentDeployment
 from dallinger.deployment import _handle_launch_data
+from dallinger.utils import develop_target_path
 from dallinger.utils import open_browser
 
 
@@ -49,13 +51,25 @@ def develop():
 
 @develop.command()
 @click.option("--port", default=5000, help="The port Flask is running on")
-def debug(port):
+@click.option(
+    "--skip-flask",
+    is_flag=True,
+    help="Skip launching Flask, so that Flask can be managed externally",
+)
+def debug(port, skip_flask):
     _bootstrap()
 
     q = Queue("default", connection=redis_conn)
-    q.enqueue_call(launch_app_and_open_dashboard, kwargs={"port": port})
+    job = q.enqueue_call(launch_app_and_open_dashboard, kwargs={"port": port})
 
-    subprocess.call(["./run.sh"], cwd="develop")
+    if not skip_flask:
+        config = get_config()
+        develop_dir = develop_target_path(config)
+        try:
+            subprocess.check_call(["./run.sh"], cwd=develop_dir)
+        except subprocess.CalledProcessError as ex:
+            job.cancel()
+            error("Failed to run flask: {} See traceback above for details.".format(ex))
 
 
 @develop.command()
@@ -65,7 +79,7 @@ def bootstrap(exp_config=None):
 
 
 def _bootstrap(exp_config=None):
-    """Creates a directory called 'develop' which will be used to host the development version of the experiment."""
+    """Creates a directory which will be used to host the development version of the experiment."""
     bootstrapper = DevelopmentDeployment(Output(), exp_config)
     log(header, chevrons=False)
     bootstrapper.run()
@@ -102,7 +116,9 @@ def _browser(route=None, port=5000):
     config.load()
     url_factory = valid_routes.get(route)
     if url_factory is not None:
-        open_browser(url_factory(config, port))
+        url = url_factory(config, port)
+        logger.info("Experiment dashboard: %s", url)
+        open_browser(url)
     else:
         click.echo(
             "Supported routes are:\n\t{}".format("\n\t".join(valid_routes.keys()))
